@@ -18,11 +18,13 @@
 #include <mstcpip.h>
 #endif
 
+#if defined(Q_OS_WIN) || defined(Q_OS_FREEBSD)
+#include <QNetworkInterface>
+#endif
+
 #include <QHostInfo>
 #include <QMetaEnum>
-#include <QNetworkConfigurationManager>
 #include <QNetworkProxy>
-#include <QNetworkSession>
 #include <QSslCipher>
 #include <QSslConfiguration>
 #include <QSslKey>
@@ -34,23 +36,25 @@
 #include "dbushelper.h"
 #include "kdeconnectconfig.h"
 #include "landevicelink.h"
-#include "qtcompat_p.h"
 
 static const int MAX_UNPAIRED_CONNECTIONS = 42;
 static const int MAX_REMEMBERED_IDENTITY_PACKETS = 42;
 
-LanLinkProvider::LanLinkProvider(bool testMode, quint16 udpBroadcastPort, quint16 udpListenPort)
+static const long MILLIS_DELAY_BETWEEN_CONNECTIONS_TO_SAME_DEVICE = 500;
+
+LanLinkProvider::LanLinkProvider(bool testMode)
     : m_server(new Server(this))
     , m_udpSocket(this)
     , m_tcpPort(0)
-    , m_udpBroadcastPort(udpBroadcastPort)
-    , m_udpListenPort(udpListenPort)
     , m_testMode(testMode)
-    , m_combineBroadcastsTimer(this)
+    , m_combineNetworkChangeTimer(this)
+#ifdef KDECONNECT_MDNS
+    , m_mdnsDiscovery(this)
+#endif
 {
-    m_combineBroadcastsTimer.setInterval(0); // increase this if waiting a single event-loop iteration is not enough
-    m_combineBroadcastsTimer.setSingleShot(true);
-    connect(&m_combineBroadcastsTimer, &QTimer::timeout, this, &LanLinkProvider::broadcastToNetwork);
+    m_combineNetworkChangeTimer.setInterval(0); // increase this if waiting a single event-loop iteration is not enough
+    m_combineNetworkChangeTimer.setSingleShot(true);
+    connect(&m_combineNetworkChangeTimer, &QTimer::timeout, this, &LanLinkProvider::combinedOnNetworkChange);
 
     connect(&m_udpSocket, &QIODevice::readyRead, this, &LanLinkProvider::udpBroadcastReceived);
 
@@ -59,21 +63,30 @@ LanLinkProvider::LanLinkProvider(bool testMode, quint16 udpBroadcastPort, quint1
 
     m_udpSocket.setProxy(QNetworkProxy::NoProxy);
 
-    // Detect when a network interface changes status, so we announce ourselves in the new network
-    QNetworkConfigurationManager *networkManager = new QNetworkConfigurationManager(this);
-    connect(networkManager, &QNetworkConfigurationManager::configurationChanged, this, &LanLinkProvider::onNetworkConfigurationChanged);
-
     connect(&m_udpSocket, &QAbstractSocket::errorOccurred, [](QAbstractSocket::SocketError socketError) {
         qWarning() << "Error sending UDP packet:" << socketError;
     });
-}
 
-void LanLinkProvider::onNetworkConfigurationChanged(const QNetworkConfiguration &config)
-{
-    if (m_lastConfig != config && config.state() == QNetworkConfiguration::Active) {
-        m_lastConfig = config;
-        onNetworkChange();
-    }
+#if QT_VERSION_MAJOR < 6
+    QNetworkConfigurationManager *networkManager = new QNetworkConfigurationManager(this);
+    connect(networkManager, &QNetworkConfigurationManager::configurationChanged, this, [this](QNetworkConfiguration config) {
+        if (config.state() == QNetworkConfiguration::Active) {
+            onNetworkChange();
+        }
+    });
+#else
+    const auto checkNetworkChange = [this]() {
+        if (QNetworkInformation::instance()->reachability() == QNetworkInformation::Reachability::Online) {
+            onNetworkChange();
+        }
+    };
+    // Detect when a network interface changes status, so we announce ourselves in the new network
+    QNetworkInformation::instance()->loadBackendByFeatures(QNetworkInformation::Feature::Reachability);
+
+    // We want to know if our current network reachability has changed, or if we change from one network to another
+    connect(QNetworkInformation::instance(), &QNetworkInformation::reachabilityChanged, this, checkNetworkChange);
+    connect(QNetworkInformation::instance(), &QNetworkInformation::transportMediumChanged, this, checkNetworkChange);
+#endif
 }
 
 LanLinkProvider::~LanLinkProvider()
@@ -84,12 +97,12 @@ void LanLinkProvider::onStart()
 {
     const QHostAddress bindAddress = m_testMode ? QHostAddress::LocalHost : QHostAddress::Any;
 
-    bool success = m_udpSocket.bind(bindAddress, m_udpListenPort, QUdpSocket::ShareAddress);
+    bool success = m_udpSocket.bind(bindAddress, UDP_PORT, QUdpSocket::ShareAddress);
     if (!success) {
         QAbstractSocket::SocketError sockErr = m_udpSocket.error();
         // Refer to https://doc.qt.io/qt-5/qabstractsocket.html#SocketError-enum to decode socket error number
         QString errorMessage = QString::fromLatin1(QMetaEnum::fromType<QAbstractSocket::SocketError>().valueToKey(sockErr));
-        qCritical(KDECONNECT_CORE) << QLatin1String("Failed to bind UDP socket on port") << m_udpListenPort << QLatin1String("with error") << errorMessage;
+        qCritical(KDECONNECT_CORE) << "Failed to bind UDP socket on port" << UDP_PORT << "with error" << errorMessage;
     }
     Q_ASSERT(success);
 
@@ -103,12 +116,20 @@ void LanLinkProvider::onStart()
         }
     }
 
-    onNetworkChange();
+    broadcastUdpIdentityPacket();
+
+#ifdef KDECONNECT_MDNS
+    m_mdnsDiscovery.onStart();
+#endif
+
     qCDebug(KDECONNECT_CORE) << "LanLinkProvider started";
 }
 
 void LanLinkProvider::onStop()
 {
+#ifdef KDECONNECT_MDNS
+    m_mdnsDiscovery.onStop();
+#endif
     m_udpSocket.close();
     m_server->close();
     qCDebug(KDECONNECT_CORE) << "LanLinkProvider stopped";
@@ -116,55 +137,38 @@ void LanLinkProvider::onStop()
 
 void LanLinkProvider::onNetworkChange()
 {
-    if (m_combineBroadcastsTimer.isActive()) {
-        qCDebug(KDECONNECT_CORE) << "Preventing duplicate broadcasts";
+    if (m_combineNetworkChangeTimer.isActive()) {
+        qCDebug(KDECONNECT_CORE) << "Device discovery triggered too fast, ignoring";
         return;
     }
-    m_combineBroadcastsTimer.start();
+    m_combineNetworkChangeTimer.start();
 }
 
 // I'm in a new network, let's be polite and introduce myself
-void LanLinkProvider::broadcastToNetwork()
+void LanLinkProvider::combinedOnNetworkChange()
 {
     if (!m_server->isListening()) {
-        // Not started
+        qWarning() << "TCP server not listening, not broadcasting";
         return;
     }
 
     Q_ASSERT(m_tcpPort != 0);
 
-    qCDebug(KDECONNECT_CORE()) << "Broadcasting identity packet";
-
-    QList<QHostAddress> destinations = getBroadcastAddresses();
-
-    NetworkPacket np = KdeConnectConfig::instance().deviceInfo().toIdentityPacket();
-    np.set(QStringLiteral("tcpPort"), m_tcpPort);
-#if defined(Q_OS_MAC) || defined(Q_OS_FREEBSD)
-    // On macOS and FreeBSD, the too large UDP packet (larger than MTU) causes
-    // incomplete transmission.
-    // We remove the capacitilities to reduce the discovery packet to the min
-    // MTU of the interfaces with broadcast feature.
-    int mtu = 1500;
-    for (const QNetworkInterface &iface : QNetworkInterface::allInterfaces()) {
-        if ((iface.flags() & QNetworkInterface::IsUp) && (iface.flags() & QNetworkInterface::IsRunning) && (iface.flags() & QNetworkInterface::CanBroadcast)) {
-            int ifaceMtu = iface.maximumTransmissionUnit();
-            if (ifaceMtu < mtu && ifaceMtu > 0) {
-                mtu = ifaceMtu;
-            }
-        }
-    }
-    QByteArray payload = np.serialize();
-    if (payload.length() > mtu) {
-        // First try to drop the less important outgoing capabilities
-        np.set(QStringLiteral("outgoingCapabilities"), QStringList());
-        payload = np.serialize();
-    }
-    if (payload.length() > mtu) {
-        // If still too large, drop the incoming capabilities
-        np.set(QStringLiteral("incomingCapabilities"), QStringList());
-        payload = np.serialize();
-    }
+    broadcastUdpIdentityPacket();
+#ifdef KDECONNECT_MDNS
+    m_mdnsDiscovery.onNetworkChange();
 #endif
+}
+
+void LanLinkProvider::broadcastUdpIdentityPacket()
+{
+    if (qEnvironmentVariableIsSet("KDECONNECT_DISABLE_UDP_BROADCAST")) {
+        qWarning() << "Not broadcasting UDP because KDECONNECT_DISABLE_UDP_BROADCAST is set";
+        return;
+    }
+    qCDebug(KDECONNECT_CORE) << "Broadcasting identity packet";
+
+    QList<QHostAddress> addresses = getBroadcastAddresses();
 
 #if defined(Q_OS_WIN) || defined(Q_OS_FREEBSD)
     // On Windows and FreeBSD we need to broadcast from every local IP address to reach all networks
@@ -175,16 +179,16 @@ void LanLinkProvider::broadcastToNetwork()
             for (const QNetworkAddressEntry &ifaceAddress : iface.addressEntries()) {
                 QHostAddress sourceAddress = ifaceAddress.ip();
                 if (sourceAddress.protocol() == QAbstractSocket::IPv4Protocol && sourceAddress != QHostAddress::LocalHost) {
-                    qCDebug(KDECONNECT_CORE()) << "Broadcasting as" << sourceAddress;
+                    qCDebug(KDECONNECT_CORE) << "Broadcasting as" << sourceAddress;
                     sendSocket.bind(sourceAddress);
-                    sendBroadcasts(sendSocket, np, destinations);
+                    sendUdpIdentityPacket(sendSocket, addresses);
                     sendSocket.close();
                 }
             }
         }
     }
 #else
-    sendBroadcasts(m_udpSocket, np, destinations);
+    sendUdpIdentityPacket(addresses);
 #endif
 }
 
@@ -211,12 +215,31 @@ QList<QHostAddress> LanLinkProvider::getBroadcastAddresses()
     return destinations;
 }
 
-void LanLinkProvider::sendBroadcasts(QUdpSocket &socket, const NetworkPacket &np, const QList<QHostAddress> &addresses)
+void LanLinkProvider::sendUdpIdentityPacket(const QList<QHostAddress> &addresses)
 {
-    const QByteArray payload = np.serialize();
+    sendUdpIdentityPacket(m_udpSocket, addresses);
+}
+
+void LanLinkProvider::sendUdpIdentityPacket(QUdpSocket &socket, const QList<QHostAddress> &addresses)
+{
+    DeviceInfo myDeviceInfo = KdeConnectConfig::instance().deviceInfo();
+    NetworkPacket identityPacket = myDeviceInfo.toIdentityPacket();
+    identityPacket.set(QStringLiteral("tcpPort"), m_tcpPort);
+    const QByteArray payload = identityPacket.serialize();
 
     for (auto &address : addresses) {
-        socket.writeDatagram(payload, address, m_udpBroadcastPort);
+        qint64 bytes = socket.writeDatagram(payload, address, UDP_PORT);
+        if (bytes == -1 && socket.error() == QAbstractSocket::DatagramTooLargeError) {
+            // On macOS and FreeBSD, UDP broadcasts larger than MTU get dropped. See:
+            // https://opensource.apple.com/source/xnu/xnu-3789.1.32/bsd/netinet/ip_output.c.auto.html#:~:text=/*%20don%27t%20allow%20broadcast%20messages%20to%20be%20fragmented%20*/
+            // We remove the capabilities to reduce the size of the packet.
+            // This should only happen for broadcasts, so UDP packets sent from MDNS discoveries should still work.
+            qWarning() << "Identity packet to" << address << "got rejected because it was too large. Retrying without including the capabilities";
+            identityPacket.set(QStringLiteral("outgoingCapabilities"), QStringList());
+            identityPacket.set(QStringLiteral("incomingCapabilities"), QStringList());
+            const QByteArray smallPayload = identityPacket.serialize();
+            socket.writeDatagram(smallPayload, address, UDP_PORT);
+        }
     }
 }
 
@@ -251,12 +274,22 @@ void LanLinkProvider::udpBroadcastReceived()
             continue;
         }
 
-        if (receivedPacket->get<QString>(QStringLiteral("deviceId")) == KdeConnectConfig::instance().deviceId()) {
+        QString deviceId = receivedPacket->get<QString>(QStringLiteral("deviceId"));
+
+        if (deviceId == KdeConnectConfig::instance().deviceId()) {
             // qCDebug(KDECONNECT_CORE) << "Ignoring my own broadcast";
             delete receivedPacket;
             continue;
         }
 
+        qint64 now = QDateTime::currentMSecsSinceEpoch();
+        if (m_lastConnectionTime[deviceId] + MILLIS_DELAY_BETWEEN_CONNECTIONS_TO_SAME_DEVICE > now) {
+            qCDebug(KDECONNECT_CORE) << "Discarding second UPD packet from the same device" << deviceId << "received too quickly";
+            delete receivedPacket;
+            return;
+        }
+        m_lastConnectionTime[deviceId] = now;
+
         int tcpPort = receivedPacket->get<int>(QStringLiteral("tcpPort"));
         if (tcpPort < MIN_TCP_PORT || tcpPort > MAX_TCP_PORT) {
             qCDebug(KDECONNECT_CORE) << "TCP port outside of kdeconnect's range";
@@ -278,11 +311,10 @@ void LanLinkProvider::udpBroadcastReceived()
         m_receivedIdentityPackets[socket].np = receivedPacket;
         m_receivedIdentityPackets[socket].sender = sender;
         connect(socket, &QAbstractSocket::connected, this, &LanLinkProvider::tcpSocketConnected);
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-        connect(socket, QOverload<QAbstractSocket::SocketError>::of(&QAbstractSocket::error), this, &LanLinkProvider::connectError);
-#else
         connect(socket, &QAbstractSocket::errorOccurred, this, &LanLinkProvider::connectError);
-#endif
+        connect(socket, &QObject::destroyed, this, [this, socket]() {
+            delete m_receivedIdentityPackets.take(socket).np;
+        });
         socket->connectToHost(sender, tcpPort);
     }
 }
@@ -297,11 +329,10 @@ void LanLinkProvider::connectError(QAbstractSocket::SocketError socketError)
     qCDebug(KDECONNECT_CORE) << "Fallback (1), try reverse connection (send udp packet)" << socket->errorString();
     NetworkPacket np = KdeConnectConfig::instance().deviceInfo().toIdentityPacket();
     np.set(QStringLiteral("tcpPort"), m_tcpPort);
-    m_udpSocket.writeDatagram(np.serialize(), m_receivedIdentityPackets[socket].sender, m_udpBroadcastPort);
+    m_udpSocket.writeDatagram(np.serialize(), m_receivedIdentityPackets[socket].sender, UDP_PORT);
 
     // The socket we created didn't work, and we didn't manage
     // to create a LanDeviceLink from it, deleting everything.
-    delete m_receivedIdentityPackets.take(socket).np;
     socket->deleteLater();
 }
 
@@ -314,11 +345,7 @@ void LanLinkProvider::tcpSocketConnected()
         return;
     }
 
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-    disconnect(socket, QOverload<QAbstractSocket::SocketError>::of(&QAbstractSocket::error), this, &LanLinkProvider::connectError);
-#else
     disconnect(socket, &QAbstractSocket::errorOccurred, this, &LanLinkProvider::connectError);
-#endif
 
     configureSocket(socket);
 
@@ -345,18 +372,17 @@ void LanLinkProvider::tcpSocketConnected()
 
         connect(socket, &QSslSocket::encrypted, this, &LanLinkProvider::encrypted);
 
-        connect(socket, QOverload<const QList<QSslError> &>::of(&QSslSocket::sslErrors), this, &LanLinkProvider::sslErrors);
+        connect(socket, &QSslSocket::sslErrors, this, &LanLinkProvider::sslErrors);
 
         socket->startServerEncryption();
     } else {
         // The socket doesn't seem to work, so we can't create the connection.
 
         qCDebug(KDECONNECT_CORE) << "Fallback (2), try reverse connection (send udp packet)";
-        m_udpSocket.writeDatagram(np2.serialize(), m_receivedIdentityPackets[socket].sender, m_udpBroadcastPort);
+        m_udpSocket.writeDatagram(np2.serialize(), m_receivedIdentityPackets[socket].sender, UDP_PORT);
 
-        // Cleanup the network packet now. The socket should be deleted via the disconnected() signal.
-        // We don't do this on success, because it is done later in the encrypted() slot.
-        delete m_receivedIdentityPackets.take(socket).np;
+        // Disconnect should trigger deleteLater, which should remove the socket from m_receivedIdentityPackets
+        socket->disconnectFromHost();
     }
 }
 
@@ -374,10 +400,11 @@ void LanLinkProvider::encrypted()
 
     DeviceInfo deviceInfo = DeviceInfo::FromIdentityPacketAndCert(*identityPacket, socket->peerCertificate());
 
-    addLink(socket, deviceInfo);
-
     // We don't delete the socket because now it's owned by the LanDeviceLink
+    disconnect(socket, &QObject::destroyed, nullptr, nullptr);
     delete m_receivedIdentityPackets.take(socket).np;
+
+    addLink(socket, deviceInfo);
 }
 
 void LanLinkProvider::sslErrors(const QList<QSslError> &errors)
@@ -397,8 +424,8 @@ void LanLinkProvider::sslErrors(const QList<QSslError> &errors)
     }
 
     if (fatal) {
+        // Disconnect should trigger deleteLater, which should remove the socket from m_receivedIdentityPackets
         socket->disconnectFromHost();
-        delete m_receivedIdentityPackets.take(socket).np;
     }
 }
 
@@ -473,6 +500,9 @@ void LanLinkProvider::dataReceived()
 
     // Needed in "encrypted" if ssl is used, similar to "tcpSocketConnected"
     m_receivedIdentityPackets[socket].np = np;
+    connect(socket, &QObject::destroyed, this, [this, socket]() {
+        delete m_receivedIdentityPackets.take(socket).np;
+    });
 
     const QString &deviceId = np->get<QString>(QStringLiteral("deviceId"));
     // qCDebug(KDECONNECT_CORE) << "Handshaking done (i'm the new device)";
@@ -488,7 +518,7 @@ void LanLinkProvider::dataReceived()
     connect(socket, &QSslSocket::encrypted, this, &LanLinkProvider::encrypted);
 
     if (isDeviceTrusted) {
-        connect(socket, QOverload<const QList<QSslError> &>::of(&QSslSocket::sslErrors), this, &LanLinkProvider::sslErrors);
+        connect(socket, &QSslSocket::sslErrors, this, &LanLinkProvider::sslErrors);
     }
 
     socket->startClientEncryption();
@@ -559,7 +589,7 @@ void LanLinkProvider::configureSocket(QSslSocket *socket)
 #endif
 
 #if defined(Q_OS_WIN)
-    int maxIdle = 5 * 60 * 1000; // 5 minutes of idle before sending keep-alives
+    int maxIdle = 5 * 60 * 1000; // 5 minutes of idle before sending keep-alive
     int interval = 5 * 1000; // 5 seconds interval between probes after 5 minute delay
     DWORD nop;
 
@@ -613,3 +643,5 @@ void LanLinkProvider::addLink(QSslSocket *socket, const DeviceInfo &deviceInfo)
     }
     Q_EMIT onConnectionReceived(deviceLink);
 }
+
+#include "moc_lanlinkprovider.cpp"
