@@ -8,8 +8,6 @@
 
 #include <KLocalizedString>
 #include <KPluginFactory>
-
-#include "plugin_lock_debug.h"
 #include <QDebug>
 
 #include <core/daemon.h>
@@ -27,10 +25,6 @@ LockDevicePlugin::LockDevicePlugin(QObject *parent, const QVariantList &args)
 {
 }
 
-LockDevicePlugin::~LockDevicePlugin()
-{
-}
-
 bool LockDevicePlugin::isLocked() const
 {
     return m_remoteLocked; // Windows doesn't support monitoring lock status, m_remoteLocked is never updated
@@ -42,7 +36,7 @@ void LockDevicePlugin::setLocked(bool locked)
     sendPacket(np);
 }
 
-bool LockDevicePlugin::receivePacket(const NetworkPacket &np)
+void LockDevicePlugin::receivePacket(const NetworkPacket &np)
 {
     if (np.has(QStringLiteral("isLocked"))) {
         bool locked = np.get<bool>(QStringLiteral("isLocked"));
@@ -81,8 +75,6 @@ bool LockDevicePlugin::receivePacket(const NetworkPacket &np)
 
         sendState();
     }
-
-    return true;
 }
 
 void LockDevicePlugin::sendState()
@@ -99,7 +91,8 @@ void LockDevicePlugin::connected()
 
 QString LockDevicePlugin::dbusPath() const
 {
-    return QStringLiteral("/modules/kdeconnect/devices/") + device()->id() + QStringLiteral("/lockdevice");
+    return QLatin1String("/modules/kdeconnect/devices/%1/lockdevice").arg(device()->id());
 }
 
 #include "lockdeviceplugin-win.moc"
+#include "moc_lockdeviceplugin-win.cpp"
