@@ -41,7 +41,7 @@ int main(int argc, char **argv)
     QApplication app(argc, argv);
     app.setWindowIcon(QIcon::fromTheme(QStringLiteral("kdeconnect")));
     const QString description = i18n("KDE Connect URL handler");
-    KAboutData about(QStringLiteral("kdeconnect-urlhandler"),
+    KAboutData about(QStringLiteral("kdeconnect.handler"),
                      description,
                      QStringLiteral(KDECONNECT_VERSION_STRING),
                      description,
@@ -113,8 +113,13 @@ int main(int argc, char **argv)
         }
     });
 
-    QObject::connect(urlRequester, &KUrlRequester::textChanged, [urlRequester, &uidialog](const QString &newUrl) {
-        bool isLocalFileUrl = QFileInfo(newUrl).exists() && QFileInfo(newUrl).isFile(); // we don't support sending directories yet!
+    QObject::connect(urlRequester, &KUrlRequester::textChanged, [&urlRequester, &uidialog]() {
+        QUrl fileUrl(urlRequester->url());
+        bool isLocalFileUrl = false;
+        if (fileUrl.isLocalFile()) {
+            QFileInfo fileInfo(fileUrl.toLocalFile());
+            isLocalFileUrl = fileInfo.exists() && fileInfo.isFile(); // we don't support sending directories yet!
+        }
         uidialog.sendFileRadioButton->setChecked(isLocalFileUrl);
         uidialog.sendUrlRadioButton->setChecked(!isLocalFileUrl);
     });
@@ -158,7 +163,7 @@ int main(int argc, char **argv)
             const QString action = open && url.isLocalFile() ? QStringLiteral("openFile") : QStringLiteral("shareUrl");
 
             QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"),
-                                                              QStringLiteral("/modules/kdeconnect/devices/") + device + QStringLiteral("/share"),
+                                                              QLatin1String("/modules/kdeconnect/devices/%1/share").arg(device),
                                                               QStringLiteral("org.kde.kdeconnect.device.share"),
                                                               action);
             msg.setArguments({url.toString()});
